@@ -103,7 +103,7 @@ show_about (JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 		  "comments", _("A GTK media player front end for xine-lib"),
 		  "license", licence,
 		  "wrap-license", TRUE,
-		  "website", "http://xinehq.de/",
+		  "website", "http://www.xine-project.org/",
 		  "authors", authors,
 		  "documenters", documenters,
 		  "translator-credits",
@@ -115,7 +115,7 @@ show_about (JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 		       VENDOR_PKG_VERSION CSET_ID,
 		       xine_get_version_string (), XINE_VERSION);
     label = ui_label_new_with_markup (version_label);
-    gtk_label_set_justify (label, GTK_JUSTIFY_CENTER);
+    gtk_label_set_justify ((GtkLabel *) label, GTK_JUSTIFY_CENTER);
     free (version_label);
     vbox = GTK_BOX (GTK_DIALOG (about)->vbox);
     gtk_box_pack_end (vbox, label, FALSE, FALSE, 2);
