@@ -108,6 +108,11 @@ static void gtk_video_size_allocate (GtkWidget      *widget,
 static gboolean gtv_unblank_screen (GtkVideo *);
 static gboolean gtv_update_resize_factor (GtkVideo *);
 
+static int screensaver_poke (const char *const [], const char *);
+#ifdef XDG_SCREENSAVER_COMMAND
+static int xdg_screensaver_poke (const char *, Window);
+#endif
+
 static GtkWidgetClass *parent_class = NULL;
 static pthread_mutex_t resize_lock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t resize_add_lock = PTHREAD_MUTEX_INITIALIZER;
@@ -185,6 +190,11 @@ struct gtk_video_private_s {
   guint32		 cookie;
 #endif
 
+#ifdef XDG_SCREENSAVER_COMMAND
+  Window		 xdg_active;
+  gboolean		 have_xdg_screensaver:8;
+#endif
+
   gboolean		 playing_no_blank:8;
 
   /* visualisation & post-plugins*/
@@ -753,7 +763,7 @@ static void frame_output_cb (void *gv_gen,
   {
     if (priv->frame_cb_id)
       g_source_remove (priv->frame_cb_id);
-    priv->frame_cb_id = g_idle_add (priv->frame_cb, priv->frame_cb_data);
+    priv->frame_cb_id = g_idle_add ((GSourceFunc) priv->frame_cb, priv->frame_cb_data);
   }
 }
 
@@ -856,7 +866,7 @@ static xine_video_port_t *load_video_out_driver (GtkVideo *this)
   x11vis.user_data       = this;
 #endif
 
-  char **choices = get_driver_ids (xine_list_video_output_plugins (priv->xine));
+  char **choices = get_driver_ids (gxine_list_video_output_plugins (priv->xine));
   /* try to init video with stored information */
   int i = xine_config_register_enum (priv->xine,
 				     "video.driver", 0, choices,
@@ -1313,6 +1323,10 @@ static void gtk_video_realize (GtkWidget *widget)
   priv->pointer_visible = TRUE;
   XUnlockDisplay (xdisplay);
 
+#ifdef XDG_SCREENSAVER_COMMAND
+  priv->xdg_active = 0;
+  priv->have_xdg_screensaver = !xdg_screensaver_poke ("--help", 0);
+#endif
   priv->screen_blanker_timeout =
     g_timeout_add (4000, (GSourceFunc) gtv_screen_blanker_cb, gtv);
 }
@@ -2207,15 +2221,52 @@ static int screensaver_poke (const char *const argv[], const char *name)
       }
     g_error_free (err);
   }
+  else
+    ret = 0;
 
   return ret;
 }
 
+#ifdef XDG_SCREENSAVER_COMMAND
+static int xdg_screensaver_poke (const char *arg, Window id)
+{
+  char id_str[16];
+  const char *const argv[] = { XDG_SCREENSAVER_COMMAND, arg,
+			       id ? id_str : NULL, NULL };
+  sprintf (id_str, "%#x", (int) id);
+  /* screensaver name is used in "can't poke <name>" */
+  return screensaver_poke (argv, "xdg-screensaver");
+}
+#endif
+
 static gboolean gtv_unblank_screen (GtkVideo *gtv)
 {
   static gboolean dbus_deny = FALSE;
+  gboolean inhibit = gtv_test_inhibit_blank (gtv);
 
-  if (!gtv_test_inhibit_blank (gtv))
+#ifdef XDG_SCREENSAVER_COMMAND
+  if (gtv->priv->have_xdg_screensaver)
+  {
+    if (inhibit != !!gtv->priv->xdg_active)
+    {
+      Window id = inhibit
+		? gdk_x11_drawable_get_xid (gtk_widget_get_toplevel (&gtv->widget)->window)
+		: gtv->priv->xdg_active;
+      int ret = 0;
+      if (!id || !(ret = xdg_screensaver_poke (inhibit ? "suspend" : "resume", id)))
+      {
+        if (ret < 0) /* if this happens, xdg-screensaver isn't there */
+          gtv->priv->have_xdg_screensaver = 0;
+        gtv->priv->xdg_active = inhibit ? id : 0;
+	return FALSE;
+      }
+    }
+    else
+      return FALSE; /* nothing to do */
+  }
+#endif
+
+  if (!inhibit)
   {
     if (dbus_deny)
       gtv_screensaver_maybe_inhibit_dbus (gtv->priv, dbus_deny = FALSE);
