@@ -344,7 +344,7 @@ static gchar* kb2str (key_binding_t *kb)
   guint              l;
   gchar             *name;
   const gchar       *key;
-  const gchar	    *key_alloc = NULL;
+  gchar		    *key_alloc = NULL;
 
   if (!text.shift)
   {
@@ -1324,7 +1324,7 @@ static JSBool js_event (JSContext *cx, JSObject *obj, uintN argc,
 static JSBool js_vdr (JSContext *cx, JSObject *obj, uintN argc,
 		      jsval *argv, jsval *rval)
 {
-  const char *const prefixes[] = { "vdr:/", "vdr-socket:/", NULL };
+  const char *const prefixes[] = { "vdr:/", "netvdr:/", NULL };
   return js_event_generic (cx, obj, argc, argv, rval, "vdr", prefixes,
 			   &xine_vdr, NULL);
 }
