@@ -31,6 +31,7 @@
 #include "utils.h"
 #include "gtkvideo.h"
 #include "vis.h"
+#include "noskin_window.h"
 
 /*
  * global variables
@@ -259,12 +260,18 @@ JS_CB (Prefs, "preferences_show ();")
 JS_CB (Keys, "keybindings_show ();")
 JS_CB (Startup, "startup_cmds_show ();")
 JS_CB (Snapshot, "snapshot ();")
+JS_CB (HideMain, "set_minimised (1);")
+JS_CB (ShowMain, "set_minimised (0);")
 JS_CB (Quit, "exit ();")
 
 /* View */
 JS_CB_T (Fullscreen)
 {
+  if (!GTK_WIDGET_MAPPED (app))
+    app_show ();
   gtk_video_set_fullscreen ((GtkVideo *)gtv, v);
+  gtk_action_set_sensitive (action_items.hidemain, !v);
+  gtk_action_set_sensitive (action_items.showmain, !v);
   if (v)
     window_fs_toolbar_restore ();
   else
@@ -330,6 +337,14 @@ JS_CB (SetupWizards, "run_wizards ();")
 JS_CB (ReloadLIRC, "reload_lirc ();")
 #endif
 
+/* Systray */
+JS_CB (PrevTrack, "playlist_play (playlist_get_item () - 1);")
+JS_CB (NextTrack, "playlist_play (playlist_get_item () + 1);")
+JS_CB (Play, "play ();")
+JS_CB (Pause, "if (!is_live_stream ()) pause ();")
+JS_CB (FFwd, "if (!is_live_stream ()) av_speed.v = 5;")
+JS_CB (Stop, "stop ();")
+
 #define CB(name, stock, label, key) \
   { #name, (stock), (label), (key), NULL, G_CALLBACK(js_##name##_cb) }
 static const GtkActionEntry main_actions[] = {
@@ -348,6 +363,8 @@ static const GtkActionEntry main_actions[] = {
   CB(Snapshot,		NULL,			N_("_Snapshot..."),		"<control>S"),
   CB(Quit,		GTK_STOCK_QUIT,		N_("_Quit"),			"<control>Q"),
   { "ViewMenu",		NULL,			N_("_View") },
+  CB(HideMain,		GTK_STOCK_CLOSE,	N_("_Hide video"),		"<control>W"),
+  CB(ShowMain,		NULL,			N_("_Show video"),		NULL),
   { "ToolbarMenu",	NULL,			N_("_Toolbars") },
   { "WindowMenu",	NULL,			N_("_Window size") },
   CB(WindowRevert,	NULL,			N_("Re_set"),			NULL),
@@ -380,6 +397,12 @@ static const GtkActionEntry main_actions[] = {
   CB(SetupWizards,	GTK_STOCK_REDO,		N_("_Re-run setup wizards..."), NULL),
   { "HelpMenu",		NULL,			N_("_Help"),			NULL },
   CB(About,		GTK_STOCK_ABOUT,	N_("_About..."),		NULL),
+  CB(PrevTrack,	GTK_STOCK_MEDIA_PREVIOUS,	N_("Pre_vious track"),		NULL),
+  CB(Play,	GTK_STOCK_MEDIA_PLAY,		N_("_Play"),			NULL),
+  CB(Pause,	GTK_STOCK_MEDIA_PAUSE,		N_("_Pause"),			NULL),
+  CB(FFwd,	GTK_STOCK_MEDIA_FORWARD,	N_("_Fast forward"),		NULL),
+  CB(Stop,	GTK_STOCK_MEDIA_STOP,		N_("_Stop"),			NULL),
+  CB(NextTrack,	GTK_STOCK_MEDIA_NEXT,		N_("_Next track"),		NULL),
 };
 #undef CB
 
@@ -500,6 +523,8 @@ static GtkWidget *add_menu_ui (GtkUIManager *ui, int i)
 	"</menu>\n"
 	"<menu action='ViewMenu'>\n"
 	  "<menuitem action='Fullscreen' />\n"
+	  "<menuitem action='HideMain' />\n"
+	  "<menuitem action='ShowMain' />\n"
 	  "<menu action='ToolbarMenu'>\n"
 	    "<menuitem action='WMToolbarVisible' />\n"
 	    "<menuitem action='WMToolbarSnap' />\n"
@@ -772,6 +797,9 @@ void create_menus (GtkWidget *window)
   action_items.subtitles = get_radio_group ("Subtitles0");
 
   action_items.keypad = get_action ("Keypad");
+  action_items.hidemain = get_action ("HideMain");
+  action_items.showmain = get_action ("ShowMain");
+  gtk_action_set_visible (action_items.showmain, FALSE);
 
   if (xine_config_lookup_entry (xine, "gui.windowed_mode.separate_toolbar",
 				&entry)
