@@ -43,6 +43,7 @@
 #include "drag_drop.h"
 #include "gtkvideo.h"
 #include "vis.h"
+#include "systray.h"
 #include "xml_widgets.h"
 
 /*
@@ -238,7 +239,7 @@ static gboolean window_fs_toolbar_show_int (void)
   window_fs_toolbar_set_geometry ();
   window_fs_toolbar_move (NULL);
   window_show (fs_toolbar, NULL);
-  if (fs_always_sticky)
+  if (fs_is_sticky)
     gtk_window_stick (GTK_WINDOW(fs_toolbar));
   XChangeProperty (GDK_WINDOW_XDISPLAY (fs_toolbar->window),
 		   GDK_WINDOW_XWINDOW (fs_toolbar->window), opacity_lock,
@@ -340,14 +341,21 @@ void window_wm_toolbar_set_snap (gboolean snap)
 
 void window_fs_stickiness_update (gboolean data)
 {
-  fs_always_sticky = data;
+  printf ("Marking as %ssticky [%ssticky]\n", data ? "" : "non-", app_prop.sticky ? "" : "non-");
+  fs_is_sticky = data;
 
   if (gtk_video_is_fullscreen ((GtkVideo *) gtv))
   {
     if (data || app_prop.sticky)
+    {
+      puts ("calling gtk_window_stick");
       gtk_window_stick ((GtkWindow *) app);
+    }
     else
+    {
+      puts ("calling gtk_window_unstick");
       gtk_window_unstick ((GtkWindow *) app);
+    }
   }
 }
 
@@ -475,7 +483,7 @@ app_state_cb (GtkWindow *window, GdkEventWindowState *event, gpointer data)
     gtk_video_deny_shrink ((GtkVideo *) gtv);
 
     /* restore stickiness as was, pre-fullscreen */
-    if (fs_always_sticky)
+    if (fs_is_sticky)
     {
       if (app_prop.sticky)
 	gtk_window_stick (window);
@@ -513,7 +521,7 @@ app_state_cb (GtkWindow *window, GdkEventWindowState *event, gpointer data)
   else
   {
     gtk_window_set_shade (window, FALSE);
-    if (fs_always_sticky)
+    if (fs_is_sticky)
       gtk_window_stick (window);
     if (!wm_toolbar)
       gtk_container_foreach (GTK_CONTAINER (window->bin.child),
@@ -532,6 +540,8 @@ app_state_cb (GtkWindow *window, GdkEventWindowState *event, gpointer data)
 
 void app_show (void)
 {
+  gtk_action_set_visible (action_items.hidemain, TRUE);
+  gtk_action_set_visible (action_items.showmain, FALSE);
   gtk_widget_show (app);
   if (wm_toolbar && wm_toolbar_visible)
     gtk_widget_show (wm_toolbar);
@@ -539,6 +549,12 @@ void app_show (void)
 
 void app_hide (void)
 {
+  /* can't do this in full-screen mode or when we have no systray */
+  if (gtk_video_is_fullscreen ((GtkVideo *)gtv) || !systray_present ())
+    return;
+
+  gtk_action_set_visible (action_items.hidemain, FALSE);
+  gtk_action_set_visible (action_items.showmain, TRUE);
   gtk_widget_hide (app);
   if (wm_toolbar && wm_toolbar_visible)
     gtk_widget_hide (wm_toolbar);
@@ -610,7 +626,7 @@ xevent_properties (void)
 
   if (app_prop.real_sticky != new_prop.real_sticky)
   {
-    if (!new_prop.fs || !fs_always_sticky)
+    if (!new_prop.fs || !fs_is_sticky)
       new_prop.sticky = new_prop.real_sticky;
     else if (!new_prop.real_sticky)
     {
@@ -621,7 +637,7 @@ xevent_properties (void)
   }
 
   if (app_prop.shaded != new_prop.shaded && wm_toolbar_snap)
-    g_idle_add (window_wm_toolbar_snap, NULL);
+    g_idle_add ((GSourceFunc) window_wm_toolbar_snap, NULL);
 
   app_prop = new_prop;
 
@@ -693,7 +709,7 @@ static gboolean noskin_main_init_cb (gpointer fullscreen)
   gdk_window_add_filter (app->window, xevent_filter_cb, app);
   gtk_widget_grab_focus (app);
 
-  gtk_video_set_frame_cb ((GtkVideo *)gtv, gxineinfo_update, timewidgets);
+  gtk_video_set_frame_cb ((GtkVideo *)gtv, (gtk_video_frame_cb_t)gxineinfo_update, timewidgets);
 
   if (fullscreen)
     gtk_window_fullscreen (GTK_WINDOW (app));
