@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2004 the xine project
+ * Copyright (C) 2003-2007 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,8 +15,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: play_item.c,v 1.48 2006/04/07 18:06:38 dsalt Exp $
- *
  * playlist item / media mark:
  * an mrl + options (e.g. volume, start time, brightness/contrast ...)
  */
@@ -43,7 +41,7 @@
 static GtkWidget *gettitle;
 
 play_item_t *play_item_new (const char *title, const char *mrl,
-			    int start_time)
+			    int start_time, int duration)
 {
   play_item_t *play_item = g_malloc0 (sizeof (play_item_t));
 
@@ -128,6 +126,7 @@ play_item_t *play_item_new (const char *title, const char *mrl,
 
   play_item->mrl        = strdup (mrl);
   play_item->start_time = start_time;
+  play_item->duration   = duration;
   play_item->options    = NULL;
 
   return play_item;
@@ -141,7 +140,7 @@ play_item_t *play_item_copy (play_item_t *src)
   if (!src)
     return NULL;
 
-  item = play_item_new (src->title, src->mrl, src->start_time);
+  item = play_item_new (src->title, src->mrl, src->start_time, src->duration);
   foreach_glist (option, src->options)
     item->options = g_list_append (item->options, strdup (option->data));
   item->untitled = src->untitled;
@@ -171,6 +170,7 @@ char *play_item_xml (const play_item_t *item, int depth)
   char *mrl = xml_escape_string (item->mrl, XML_ESCAPE_DOUBLE_QUOTE);
   char *indent = malloc (depth * 2 + 1);
   char *xml;
+  char *sub = NULL;
 
   memset (indent, ' ', depth * 2);
   indent[depth * 2] = 0;
@@ -181,20 +181,38 @@ char *play_item_xml (const play_item_t *item, int depth)
 		indent,
 		indent, item->untitled ? " default=\"y\"" : "", title,
 		indent, mrl);
+
   if (item->start_time)
   {
     char timestring[64];
     int_to_timestring (item->start_time, timestring, sizeof (timestring));
-    asreprintf (&xml, "%s>\n"
-		"%s    <STARTTIME VALUE=\"%s\" />\n"
-		"%s  </REF>\n"
-		"%s</ENTRY>\n",
-		xml, indent, timestring, indent, indent);
+    asreprintf (&sub, "%s%s    <STARTTIME VALUE=\"%s\" />\n",
+		sub ? : "", indent, timestring);
   }
+
+  if (item->duration > 0)
+  {
+    char timestring[64];
+    int_to_timestring (item->duration, timestring, sizeof (timestring));
+    asreprintf (&sub, "%s%s    <DURATION VALUE=\"%s\" />\n",
+		sub ? : "", indent, timestring);
+  }
+
+  if (item->av.set)
+    asreprintf (&sub, "%s%s    <AVOFFSET VALUE=\"%d\" />\n",
+		sub ? : "", indent, item->av.offset);
+
+  if (item->spu.set)
+    asreprintf (&sub, "%s%s    <SPUOFFSET VALUE=\"%d\" />\n",
+		sub ? : "", indent, item->spu.offset);
+
+  if (sub)
+    asreprintf (&xml, "%s>\n%s%s  </REF>\n%s</ENTRY>\n", xml, sub, indent, indent);
   else
-    asreprintf (&xml, "%s/>\n%s</ENTRY>\n", xml, indent);
+    asreprintf (&xml, "%s />\n%s</ENTRY>\n", xml, indent);
 
   free (title);
+  free (sub);
   free (mrl);
   free (indent);
 
@@ -213,6 +231,7 @@ play_item_t *play_item_load (xml_node_t *node)
   play_item->options    = NULL;
   play_item->mrl        = NULL;
   play_item->start_time = -1;
+  play_item->duration   = -1;
 
   /* assumption: node is the first child of an ENTRY node */
 
@@ -242,8 +261,8 @@ play_item_t *play_item_load (xml_node_t *node)
 	{
 	  if (!strcasecmp (asx_sub->name, "STARTTIME"))
 	    play_item->start_time = GET_TIME (asx_sub);
-	  //else if (!strcasecmp (asx_sub->name, "DURATION"))
-	    //duration = GET_TIME (asx_sub);
+	  else if (!strcasecmp (asx_sub->name, "DURATION"))
+	    play_item->duration = GET_TIME (asx_sub);
 	}
       }
     }
@@ -253,7 +272,7 @@ play_item_t *play_item_load (xml_node_t *node)
       if (!play_item->title)
       {
 	play_item->untitled = !!xml_parser_get_property (node, "default");
-	play_item->title = strdup (node->data);
+	play_item->title = strdup (node->data ? : "");
 	logprintf ("play_item: title = %s\n", play_item->title);
       }
     }
@@ -268,19 +287,23 @@ play_item_t *play_item_load (xml_node_t *node)
       }
     }
 
+    else if (!strcasecmp (node->name, "DURATION"))
+    {
+      if (play_item->duration < 0)
+      {
+	play_item->duration = GET_TIME (node);
+	logprintf ("play_item: duration = %lf s\n",
+		   play_item->duration / 1000.0);
+      }
+    }
+
     else if (!strcasecmp (node->name, "TIME"))
     {
       /* DEPRECATED; for compatibility with older versions of gxine */
       play_item->start_time = xml_parser_get_property_int (node, "START", 0);
-      logprintf ("play_item: start time = %lf s\n", play_item->start_time);
+      logprintf ("play_item: start time = %d s\n", play_item->start_time);
     }
 /*
-    else if (!strcasecmp (node->name, "DURATION"))
-    {
-      if (duration < 0)
-	duration = asx_get_time_value (node);
-    }
-
     else if (!strcasecmp (node->name, "BASE"))
       ref_base_href = xml_parser_get_property (asx_entry, "HREF");
 */
@@ -291,7 +314,8 @@ play_item_t *play_item_load (xml_node_t *node)
   return play_item;
 }
 
-void play_item_play (play_item_t *play_item, int pos, int pos_time)
+void play_item_play (play_item_t *play_item,
+		     int pos, int pos_time, int duration)
 {
   GList *option;
   int count = -1;
@@ -309,13 +333,14 @@ void play_item_play (play_item_t *play_item, int pos, int pos_time)
   }
 
   player_launch (play_item->title, play_item->mrl, pos,
-		 pos_time >= 0 ? pos_time : play_item->start_time);
+		 pos_time >= 0 ? pos_time : play_item->start_time,
+		 duration > 0 ? duration : play_item->duration);
 }
 
 /* gui part */
 
 static GtkWidget *dlg;
-static GtkWidget *entry, *mrl_entry, *time_entry;
+static GtkWidget *entry, *mrl_entry, *time_entry, *duration_entry;
 static int        is_visible;
 
 static play_item_t *edit_item = NULL;
@@ -330,6 +355,9 @@ static void play_item_set_text (void)
   int_to_timestring (round_second (edit_item->start_time),
 		     time_str, sizeof (time_str));
   gtk_entry_set_text (GTK_ENTRY (time_entry), time_str);
+  int_to_timestring (edit_item->duration < 0 ? 0 : edit_item->duration,
+		     time_str, sizeof (time_str));
+  gtk_entry_set_text (GTK_ENTRY (duration_entry), time_str);
 }
 
 static gboolean close_cb (GtkWidget* widget, gpointer data)
@@ -360,6 +388,8 @@ static void response_cb (GtkDialog *dbox, int response, gpointer data)
     edit_item->title = strdup(tmp);
     edit_item->start_time = parse_timestring
 			      (gtk_entry_get_text (GTK_ENTRY (time_entry)));
+    edit_item->duration = parse_timestring
+			    (gtk_entry_get_text (GTK_ENTRY (duration_entry)));
     edited = 1;
   default:
     free (item_title);
@@ -399,7 +429,7 @@ void play_item_init (void)
 	NULL);
   ui_add_undo_response (dlg, NULL);
 
-  table = gtk_table_new (3, 3, FALSE);
+  table = gtk_table_new (4, 3, FALSE);
 
   add_table_row_items (table, 0, FALSE, _("MRL"),
 		       mrl_entry = gtk_entry_new (), NULL);
@@ -407,14 +437,18 @@ void play_item_init (void)
 		       entry = gtk_entry_new (),
 		       gettitle = gtk_button_new_with_label (_("From stream")),
 		       NULL);
-  add_table_row_items (table, 2, FALSE, _("Time"),
+  add_table_row_items (table, 2, FALSE, _("Start time"),
 		       time_entry = gtk_entry_new (), NULL);
+  add_table_row_items (table, 3, FALSE, _("Duration"),
+		       duration_entry = gtk_entry_new (), NULL);
   gtk_entry_set_activates_default (GTK_ENTRY(mrl_entry), TRUE);
   gtk_entry_set_activates_default (GTK_ENTRY(entry), TRUE);
   gtk_entry_set_activates_default (GTK_ENTRY(time_entry), TRUE);
+  gtk_entry_set_activates_default (GTK_ENTRY(duration_entry), TRUE);
   g_object_set_data (G_OBJECT (mrl_entry), "mrl_component", "M");
   g_object_set_data (G_OBJECT (entry), "mrl_component", "N");
   g_object_set_data (G_OBJECT (time_entry), "mrl_component", "T");
+  g_object_set_data (G_OBJECT (duration_entry), "mrl_component", "D");
 
   g_signal_connect (G_OBJECT (gettitle), "clicked",
 		    (GCallback) get_title_cb, NULL);
@@ -440,10 +474,11 @@ int play_item_edit (play_item_t *item, play_item_type_t type,
   gtk_widget_set_sensitive (gettitle, item_title && *item_title);
   play_item_set_text ();
 
-  gtk_window_set_title (GTK_WINDOW(dlg), gettext (title[type]));
-  gtk_window_set_modal (GTK_WINDOW(dlg), TRUE);
+  GtkWindow *w = GTK_WINDOW (dlg);
+  gtk_window_set_title (w, gettext (title[type]));
+  gtk_window_set_modal (w, TRUE);
   window_show (dlg, parent);
-  gtk_window_set_focus (GTK_WINDOW (dlg), entry);
+  gtk_window_set_focus (w, entry);
 
   gtk_main();
 
@@ -483,10 +518,11 @@ static void clip_get_cb (GtkClipboard *cb, GtkSelectionData *sel, guint info,
     break;
   case 2:
     {
-      char timestring[64], *text;
-      int_to_timestring (item->start_time, timestring, sizeof (timestring));
-      text = g_strdup_printf (_("Title: %s\nMRL: %s\nStart: %s\n"),
-			      item->title, item->mrl, timestring);
+      char start[64], length[64], *text;
+      int_to_timestring (item->start_time, start, sizeof (start));
+      int_to_timestring (item->start_time, length, sizeof (length));
+      text = g_strdup_printf (_("Title: %s\nMRL: %s\nStart: %s\nDuration: %s"),
+			      item->title, item->mrl, start, length);
       gtk_selection_data_set_text (sel, text, -1);
       free (text);
     }
@@ -519,10 +555,12 @@ void clip_set_play_item_from_selection (GtkWidget *view)
       gtk_clipboard_set_with_data (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD),
 				   targets, G_N_ELEMENTS (targets),
 				   clip_get_cb, clip_clear_cb, item);
+/*
       gtk_clipboard_set_with_data (gtk_clipboard_get (GDK_SELECTION_PRIMARY),
 				   targets, G_N_ELEMENTS (targets),
 				   clip_get_cb, clip_clear_cb,
 				   play_item_copy (item));
+*/
     }
     g_value_unset (&vi);
   }
