@@ -83,7 +83,7 @@ static void play_exec_error (void)
   switch (error = xine_get_error (stream))
   {
   case XINE_ERROR_NO_INPUT_PLUGIN:
-    display_error (FROM_XINE, _("The xine engine failed to start."),
+    display_error (FROM_XINE_COMBINABLE, _("The xine engine failed to start."),
 		   _("No input plugin was found.\n"
 		     "Maybe the file does not exist or cannot be accessed, "
 		     "or there is an error in the URL."));
@@ -92,7 +92,7 @@ static void play_exec_error (void)
   case XINE_ERROR_NO_DEMUX_PLUGIN:
     if (playlist_showing_logo ())
     {
-      g_idle_add (infobar_show_metadata, infobars);
+      g_idle_add ((GSourceFunc)infobar_show_metadata, infobars);
       if (demuxer_for_logo)
       {
 	demuxer_for_logo = FALSE;
@@ -118,7 +118,7 @@ static void play_exec_error (void)
     break;
 
   case XINE_ERROR_INPUT_FAILED:
-    display_error (FROM_XINE, _("The xine engine failed to start."),
+    display_error (FROM_XINE_COMBINABLE, _("The xine engine failed to start."),
 		   _("The stream could not be opened."));
     break;
 
@@ -225,7 +225,7 @@ static __attribute__ ((noreturn)) void *play_exec (void *data)
 
 //    xine_play (stream, 0, 0);
     xine_play (stream, goto_pos, goto_time);
-    gtk_video_set_end_time (gtv, end_time);
+    gtk_video_set_end_time ((GtkVideo *)gtv, end_time);
     goto_pos = 0;
     goto_time = 0;
     end_time = 0;
@@ -1052,7 +1052,7 @@ static xine_audio_port_t *load_audio_out_driver (void)
   int i = xine_config_register_enum (xine,
 				     "audio.driver", 0, choices,
 				     _("audio driver to use"),
-				     NULL, 10, gtk_true, NULL);
+				     NULL, 10, GXINE_TRUE, NULL);
   if (!audio_driver_id)
     audio_driver_id = choices[i];
 
@@ -1088,7 +1088,7 @@ static void add_chain_funcs (const char *prop, const char *config,
     se_defun (gse, obj, "set_chain", set_chain, 0, 0, SE_GROUP_HIDDEN, NULL, NULL);
   se_prop_create_xine_id (gse, obj, "chain", config);
   se_prop_add_listener (gse, obj, "chain",
-			listen_postproc_chain, config);
+			listen_postproc_chain, (void *)config);
 }
 
 void player_init (void)
@@ -1162,7 +1162,9 @@ void player_init (void)
       { NULL }
     };
     static const ui_property_t fs_subprops[] = {
-      { "sticky", N_("v=bool, toggle(); forced stickiness in full-screen mode"), UI_BOOL (fs_sticky, fs_always_sticky, NULL) },
+      { "sticky", NULL, UI_BOOL (fs_sticky, fs_always_sticky, NULL) },
+        /* ↑ deprecated; it's really just gui.fullscreen_mode.always_sticky */
+      { "is_sticky", N_("v=bool, toggle(); forced stickiness in full-screen mode"), UI_BOOL (fs_sticky, fs_is_sticky, window_fs_stickiness_update) },
       { NULL }
     };
 
@@ -1268,7 +1270,7 @@ void player_init (void)
        _("Double the size of small video streams"),
        _("This affects video streams whose dimensions are at most ⅓ "
 	 "of those of the display."),
-       0, gtk_true, NULL);       
+       0, GXINE_TRUE, NULL);       
     config_update_default ("gui.windowed_mode.magnify_lowres_video", 1);
     xine_config_register_bool
       (xine, "gui.windowed_mode.auto_resize", 1,
@@ -1279,6 +1281,6 @@ void player_init (void)
       (xine, "gui.subtitle_autoload", 1,
        _("Subtitle autoloading"),
        _("Automatically load subtitles if they exist."),
-       0, gtk_true, NULL);
+       0, GXINE_TRUE, NULL);
   }
 }
