@@ -959,7 +959,7 @@ pl_smil_fetch (pl_xml_info_t *info, void *data)
 	  strcmp (info->node->name, "VIDEO"))
 	goto next;
 
-      char *mrl = xml_parser_get_property (info->node, "SRC");
+      const char *mrl = xml_parser_get_property (info->node, "SRC");
       if (mrl)
 	return play_item_new (NULL, mrl, 0, 0);
     }
@@ -1547,7 +1547,7 @@ static gboolean row_clicked_lcb (GtkWidget *widget, GdkEventButton *ev,
   return list_menu_click_cb (tree_view, ev, (GtkMenu **) data);
 }
 
-static gboolean key_press_lcb (GtkWidget *widget, GdkEventButton *ev,
+static gboolean key_press_lcb (GtkWidget *widget, GdkEventKey *ev,
 				 gpointer data)
 {
   return list_menu_key_press_cb (tree_view, ev, (GtkMenu **) data);
@@ -1558,13 +1558,46 @@ static int has_been_played (int pos)
   return playlist_get_item (pos)->played;
 }
 
+static int is_ignored (int pos)
+{
+  return playlist_get_item (pos)->ignore;
+}
+
+static int select_next_random (void)
+{
+  int pos = rand() % playlist_size();
+  int count = 0;
+  const int size = playlist_size ();
+  while (count < size && has_been_played (pos))
+  {
+    pos = (pos + 1) % size;
+    count++;
+  }
+  return has_been_played (cur_list_pos) ? size + 1 : pos;
+}
+
+static int select_next_linear (int pos)
+{
+  int count = 0;
+  const int size = playlist_size ();
+  while (count < size && is_ignored (pos))
+  {
+    pos = (pos + 1) % size;
+    count++;
+  }
+  return is_ignored (cur_list_pos) ? size + 1 : pos;
+}
+
 static void play_next (void)
 {
   int is_repeat, is_random;
 
   if (gtk_toggle_button_get_active (repeat_track_button))
   {
-    playlist_play (cur_list_pos);
+    if (!is_ignored (cur_list_pos))
+      playlist_play (cur_list_pos);
+    else
+      playlist_logo (NULL);
     return;
   }
 
@@ -1574,20 +1607,8 @@ static void play_next (void)
   unmark_play_item ();
   ref_list_pos = -1;
 
-  if (is_random)
-  {
-    cur_list_pos = rand() % playlist_size();
-    int count = 0;
-    while ((count<playlist_size()) && has_been_played (cur_list_pos))
-    {
-      cur_list_pos = (cur_list_pos + 1) % playlist_size();
-      count++;
-    }
-
-    if (has_been_played (cur_list_pos))
-      cur_list_pos = playlist_size()+1;
-  } else
-    cur_list_pos++;
+  cur_list_pos = is_random ? select_next_random ()
+			   : select_next_linear (cur_list_pos + 1);
 
   if (is_repeat && (cur_list_pos>=playlist_size()))
   {
@@ -1599,12 +1620,12 @@ static void play_next (void)
       {
 	play_item_t *item;
 	item = playlist_get_item (i);
-	item->played = 0;
+	item->played = item->ignore;
       }
 
-      cur_list_pos = rand() % playlist_size();
+      cur_list_pos = select_next_random ();
     } else
-      cur_list_pos = 0;
+      cur_list_pos = select_next_linear (0);
   }
 
   if (cur_list_pos<playlist_size())
@@ -1623,17 +1644,25 @@ static void play_next (void)
 }
 
 #ifdef XINE_MSG_AUTHENTICATION_NEEDED
-static gboolean auth_validate (const char *text)
+static char *auth_escape (GtkEntry *entry)
 {
-  if (!text || !*text)
-    return FALSE;
+  const char *text = gtk_entry_get_text (entry);
+  char *dest = malloc (strlen (text) * 3 + 1);
+  char *ptr = dest;
+
   while (*text)
   {
     unsigned char c = (unsigned char) *text++;
-    if (c < 33 || c == '/' || c == ':' || c == '@')
-      return FALSE;
+    if (c < 33 || c == '/' || c == ':' || c == '@' || c > '~')
+    {
+      sprintf (ptr, "%%%02X", c);
+      ptr += 3;
+    }
+    else
+      *ptr++ = c;
   }
-  return TRUE;
+  *ptr = 0;
+  return dest;
 }
 
 static int display_auth (int args, const char *mrl)
@@ -1657,17 +1686,12 @@ static int display_auth (int args, const char *mrl)
 
   if (gtk_dialog_run (auth.dbox) == GTK_RESPONSE_OK)
   {
-    const char *name = gtk_entry_get_text (auth.name);
-    const char *pass = gtk_entry_get_text (auth.pass);
-    if (!auth_validate (name) || !auth_validate (pass))
-    {
-      display_error (FROM_GXINE, "%s", _("Authentication data contains invalid characters."));
-      goto ret;
-    }
+    char *name = auth_escape (auth.name);
+    char *pass = auth_escape (auth.pass);
 
     host += 3;
     const char *at = strchr (host, '@');
-    const char *slash = strchr (host, ':');
+    const char *slash = strchr (host, '/');
     if (at && slash && at > slash)
       at = NULL;
     if (at)
@@ -1682,6 +1706,9 @@ static int display_auth (int args, const char *mrl)
     else
       txt = g_strdup_printf ("%.*s%s:%s@%s", (int)(host - mrl), mrl, name, pass, host);
 
+    free (name);
+    free (pass);
+
     play_item_t *play_item;
     int pos;
     for (pos = 0; pos < playlist_size (); ++pos)
@@ -1703,6 +1730,17 @@ static int display_auth (int args, const char *mrl)
     playlist_play (pos);
     retval = 1;
   }
+  else
+  {
+    play_item_t *play_item;
+    int pos;
+    for (pos = 0; pos < playlist_size (); ++pos)
+      if (!strcmp ((play_item = playlist_get_item (pos))->mrl, mrl))
+      {
+        play_item->played = play_item->ignore = TRUE;
+        break;
+      }
+  }
 
 ret:
   gtk_widget_hide ((GtkWidget *) auth.dbox);
@@ -1873,29 +1911,37 @@ typedef struct {
         g_print (_("xine_event_cb: unknown XINE_MSG_*\n"));
       case XINE_MSG_NO_ERROR:
       case XINE_MSG_AUDIO_OUT_UNAVAILABLE:
-      case XINE_MSG_ENCRYPTED_SOURCE:
         msg = display_info;
         break;
+      
+      case XINE_MSG_ENCRYPTED_SOURCE:
+        msg = display_info_combinable;
+        break;
+
+      case XINE_MSG_GENERAL_WARNING:
+      case XINE_MSG_NETWORK_UNREACHABLE:
+        msg = display_warning;
+        break;
 
       case 13 /* UNCOMMENTME: XINE_MSG_FILE_EMPTY */:
         fallback = _("File is empty:"); /* missing in xine-lib < 1.1.2 */
-      case XINE_MSG_GENERAL_WARNING:
-      case XINE_MSG_NETWORK_UNREACHABLE:
       case XINE_MSG_UNKNOWN_HOST:
-        msg = display_warning;
+        msg = display_warning_combinable;
         break;
 
-      case XINE_MSG_READ_ERROR:
       case XINE_MSG_LIBRARY_LOAD_ERROR:
-
-      case XINE_MSG_CONNECTION_REFUSED:
-      case XINE_MSG_FILE_NOT_FOUND:
-      case XINE_MSG_PERMISSION_ERROR:
       case XINE_MSG_SECURITY:
       case XINE_MSG_UNKNOWN_DEVICE:
         msg = display_error;
         break;
 
+      case XINE_MSG_READ_ERROR:
+      case XINE_MSG_CONNECTION_REFUSED:
+      case XINE_MSG_FILE_NOT_FOUND:
+      case XINE_MSG_PERMISSION_ERROR:
+        msg = display_error_combinable;
+        break;
+
 #ifdef XINE_MSG_AUTHENTICATION_NEEDED
       /* new in xine-lib 1.2 */
       case XINE_MSG_AUTHENTICATION_NEEDED:
@@ -1927,7 +1973,7 @@ typedef struct {
 	char *txt = strdup ("");
 	for (i = 0; i < data->num_parameters; ++i)
 	{
-	  asreprintf (&txt, "%s %s", txt, param);
+	  asreprintf (&txt, "%s%s%s", txt, " " + !i, param);
 	  param += strlen (param) + 1;
 	}
 	msg (FROM_XINE,
@@ -2546,6 +2592,8 @@ void playlist_init (void)
    */
   xine_event_create_listener_thread (xine_event_new_queue (stream),
 				     xine_event_cb, NULL);
+  /* give the thread a chance to start now */
+  nanosleep (&(struct timespec){ 0, 1000 }, NULL);
 
   {
     static const se_f_def_t defs[] = {
