@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2006 the xine-project
+ * Copyright (C) 2004-2007 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,8 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: post.c,v 1.45 2006/04/09 14:27:29 dsalt Exp $
- *
  * post-plugin handling
  */
 
@@ -62,6 +60,7 @@ struct gxine_post_s {
   xine_post_in_t *info;
   GtkWidget *box, *enabled, *pref, *up;	/* in parent dbox */
   GtkWidget *dbox, **conf_w;	/* this plugin's dbox */
+  guint pref_toggled;		/* signal id */
   char *params;			/* raw config */
   void (*reconfigure)(struct gxine_post_s *);
 };
@@ -372,6 +371,17 @@ post_config_post_dispose_dbox (gxine_post_t *info)
   info->conf_w = NULL;
 }
 
+static gboolean
+post_config_post_window (GtkButton *widget, gxine_post_t *info);
+
+static void
+toggle_pref_button (const gxine_post_t *info, gboolean state)
+{
+  g_signal_handler_block (G_OBJECT (info->pref), info->pref_toggled);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (info->pref), state);
+  g_signal_handler_unblock (G_OBJECT (info->pref), info->pref_toggled);
+}
+
 static void
 post_config_post_response (GtkDialog *dbox, gint response, gxine_post_t *info)
 {
@@ -388,8 +398,7 @@ post_config_post_response (GtkDialog *dbox, gint response, gxine_post_t *info)
     /* fall through */
   default:
     post_config_post_dispose_dbox (info);
-    if (info->pref)
-      gtk_widget_set_sensitive (info->pref, TRUE);
+    toggle_pref_button (info, FALSE);
     break;
   }
 }
@@ -467,7 +476,7 @@ post_config_post_reconfigure (gxine_post_t *info)
   info->parameters = params;
 }
 
-static JSBool
+static gboolean
 post_config_post_window (GtkButton *widget, gxine_post_t *info)
 {
   const xine_post_api_t *api;
@@ -479,6 +488,7 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
 
   if (info->dbox)
   {
+    toggle_pref_button (info, TRUE);
     gtk_window_present (GTK_WINDOW(info->dbox));
     return JS_FALSE;
   }
@@ -495,14 +505,14 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
 				 GTK_STOCK_APPLY, GTK_RESPONSE_APPLY,
 				 GTK_STOCK_OK, GTK_RESPONSE_OK,
 				 NULL);
+  GtkDialog *dbox = GTK_DIALOG (info->dbox);
   g_signal_connect (G_OBJECT(info->dbox), "response",
 		    G_CALLBACK(post_config_post_response), info);
-  gtk_dialog_set_default_response (GTK_DIALOG (info->dbox), GTK_RESPONSE_OK);
+  gtk_dialog_set_default_response (dbox, GTK_RESPONSE_OK);
   ui_add_undo_response (info->dbox, NULL);
   free (title);
   table = gtk_table_new (1, 2, FALSE);
-  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(info->dbox)->vbox), table,
-		      FALSE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX(dbox->vbox), table, FALSE, TRUE, 0);
 
   info->params = post_parse_parameters (info->plugin, info->parameters);
   param = param_desc->parameter - 1;
@@ -531,10 +541,7 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
       {
 	GtkObject *adj =
 	  gtk_adjustment_new (*(int *)param_p, INT_MIN, INT_MAX, 1, 10, 0);
-	widget = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1, 0);
-	gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON(widget),
-					   GTK_UPDATE_ALWAYS);
-	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON(widget), TRUE);
+	widget = ui_spin_button_new (adj);
 	g_signal_connect (G_OBJECT(widget), "value-changed",
 			  G_CALLBACK(spin_cb), param_p);
       }
@@ -552,11 +559,8 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
 	adj = gtk_adjustment_new (*(double *)param_p,
 				  param->range_min, param->range_max,
 				  step, page, 0);
-	widget = gtk_hscale_new (GTK_ADJUSTMENT(adj));
+	widget = ui_hscale_new (adj, GTK_POS_TOP, floating ? 2 : 0);
 	gtk_widget_set_size_request (widget, 100, -1);
-	gtk_scale_set_draw_value (GTK_SCALE(widget), TRUE);
-	gtk_scale_set_value_pos (GTK_SCALE(widget), GTK_POS_TOP);
-	gtk_scale_set_digits (GTK_SCALE(widget), floating ? 2 : 0); /* hmm... */
 	g_signal_connect (G_OBJECT (adj), "value-changed",
 			  G_CALLBACK (range_cb), param_p);
       }
@@ -592,8 +596,10 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
     gtk_table_attach (GTK_TABLE(table), widget,
 		      1, 2, param_count, param_count + 1, 7, 7, 2, 2);
 
-    widget = gtk_label_new (param->name);
-    gtk_misc_set_alignment (GTK_MISC (widget), 1, 0.5);
+    if (param->description)
+      gtk_tooltips_set_tip (tips, widget, param->description, NULL);
+
+    widget = ui_label_new_with_xalign (param->name, 1);
     gtk_table_attach (GTK_TABLE(table), widget,
 		      0, 1, param_count, param_count + 1,
 		      GTK_FILL, GTK_FILL, 2, 2);
@@ -664,12 +670,11 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
 				      GTK_POLICY_AUTOMATIC);
 
       gtk_widget_set_size_request (window, 1, -1);
-      gtk_box_pack_end (GTK_BOX(GTK_DIALOG(info->dbox)->vbox), window,
-			TRUE, TRUE, 2);
+      gtk_box_pack_end (GTK_BOX(dbox->vbox), window, TRUE, TRUE, 2);
     }
     free (helptext);
 
-    gtk_box_pack_end (GTK_BOX(GTK_DIALOG(info->dbox)->vbox),
+    gtk_box_pack_end (GTK_BOX(dbox->vbox),
 		      gtk_hseparator_new (), FALSE, TRUE, 2);
   }
 
@@ -680,12 +685,10 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
     GdkGeometry geom;
     geom.width_inc = 0;
     geom.height_inc = 1;
-    gtk_window_set_geometry_hints (GTK_WINDOW(info->dbox), info->dbox, &geom,
+    gtk_window_set_geometry_hints (GTK_WINDOW(dbox), info->dbox, &geom,
 				   GDK_HINT_RESIZE_INC);
   }
 
-  if (info->pref)
-    gtk_widget_set_sensitive (info->pref, FALSE);
   return JS_TRUE;
 }
 
@@ -877,11 +880,13 @@ post_config_post_add_plugin (gxine_chain_t *chain, xine_post_t *post,
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), enabled);
   gtk_box_pack_start (GTK_BOX(info->box), w, FALSE, TRUE, 0);
 
+  GtkBox *box = GTK_BOX (info->box);
+
   /* FIXME but not until combo box menu items can have tooltips */
   w = gtk_option_menu_new ();
   gtk_option_menu_set_menu (GTK_OPTION_MENU(w), m);
   gtk_option_menu_set_history (GTK_OPTION_MENU(w), history);
-  gtk_box_pack_start_defaults (GTK_BOX(info->box), w);
+  gtk_box_pack_start_defaults (box, w);
   g_signal_connect (G_OBJECT(w), "changed",
 		    G_CALLBACK(post_config_post_chain_set_plugin), info);
   if (hist_desc)
@@ -889,21 +894,23 @@ post_config_post_add_plugin (gxine_chain_t *chain, xine_post_t *post,
 
   info->up = w = ui_button_new_stock (GTK_STOCK_GO_UP);
   gtk_widget_set_sensitive (w, !!chain->plugins); /* disable for first plugin */
-  gtk_box_pack_end (GTK_BOX(info->box), w, FALSE, TRUE, 0);
+  gtk_box_pack_end (box, w, FALSE, TRUE, 0);
   g_signal_connect (G_OBJECT(w), "clicked",
 		    G_CALLBACK(post_config_post_chain_move_plugin), info);
 
   w = gtk_button_new_from_stock (GTK_STOCK_REMOVE);
-  gtk_box_pack_end (GTK_BOX(info->box), w, FALSE, TRUE, 0);
+  gtk_box_pack_end (box, w, FALSE, TRUE, 0);
   g_signal_connect (G_OBJECT(w), "clicked",
 		    G_CALLBACK(post_config_post_chain_delete_plugin), info);
 
-  info->pref = w = gtk_button_new_from_stock (GTK_STOCK_PROPERTIES);
-  gtk_box_pack_end (GTK_BOX(info->box), w, FALSE, TRUE, 0);
+  info->pref = w = gtk_toggle_button_new_with_label (GTK_STOCK_PROPERTIES);
+  g_object_set (G_OBJECT(w), "use-stock", TRUE, NULL);
+  gtk_box_pack_end (box, w, FALSE, TRUE, 0);
   if (!xine_post_input (post, "parameters"))
     gtk_widget_set_sensitive (w, FALSE);
-  g_signal_connect (G_OBJECT(w), "clicked",
-		    G_CALLBACK(post_config_post_window), info);
+  info->pref_toggled = g_signal_connect (G_OBJECT(w), "toggled",
+					 G_CALLBACK(post_config_post_window),
+					 info);
 
   gtk_box_pack_start (GTK_BOX(GTK_DIALOG(chain->dbox)->vbox), info->box,
 		      FALSE, TRUE, 2);
@@ -1024,7 +1031,7 @@ post_config_post_chain_window (gxine_chain_t *chain)
   plugin_list = xine_list_post_plugins_typed (xine, post_type[chain->type]);
   if (!plugin_list || !plugin_list[0])
   {
-    title = g_markup_printf_escaped (_("Plugin chain %s"),
+    title = g_markup_printf_escaped (_("Plugin chain: %s"),
 				     gettext (chain_title[chain->type]));
     display_info (FROM_XINE, title,
 		  _("No available plugins - nothing to configure"));
@@ -1039,7 +1046,7 @@ post_config_post_chain_window (gxine_chain_t *chain)
 
   logprintf ("post_config: start\n");
 
-  title = g_strdup_printf (_("Plugin chain %s"),
+  title = g_strdup_printf (_("Plugin chain: %s"),
 			   gettext (chain_title[chain->type]));
   chain->dbox = gtk_dialog_new_with_buttons
 		  (title, NULL, 0,
