@@ -56,6 +56,7 @@ gboolean wm_toolbar_visible = TRUE;
 gboolean wm_toolbar_snap = TRUE;
 
 gboolean fs_always_sticky = FALSE;
+gboolean fs_is_sticky = FALSE;
 
 GtkIconSize icon_size_logo;
 
@@ -689,7 +690,7 @@ void ui_create_properties (const ui_property_t *prop, se_o_t *parent,
       se_prop_add_listener (gse, obj, "v", prop[i].listen.cb,
 			    prop[i].listen.user_data);
     else
-      se_prop_add_listener (gse, obj, "v", cb, &prop[i]);
+      se_prop_add_listener (gse, obj, "v", cb, (void *)&prop[i]);
 
     switch (type)
     {
@@ -757,7 +758,7 @@ void ui_preferences_register (xine_t *this)
     (this, "gui.experience_level", 0, experience_labels,
      _("Display of configuration settings"),
      _("Controls whether more advanced configuration settings are shown."),
-     0, gtk_true, NULL);
+     0, GXINE_TRUE, NULL);
 
   /* Register a few audio/video config items */
   for (i = 0; i < G_N_ELEMENTS (ranges); ++i)
@@ -825,9 +826,9 @@ void ui_preferences_register (xine_t *this)
   xine_config_register_bool
     (this, "gui.post_plugins.always_show_logo", 1,
      _("In windowed mode, whether the logo is shown after playing an audio-only stream if no audio visualisation was active."), NULL,
-     0, gtk_true, NULL);
+     0, GXINE_TRUE, NULL);
 
-  fs_always_sticky = xine_config_register_bool
+  fs_is_sticky = fs_always_sticky = xine_config_register_bool
     (this, "gui.fullscreen_mode.always_sticky", 1,
      _("In full-screen mode, whether the video always appears on all desktops & viewports"),
      NULL, 0, NULL, NULL);
@@ -1101,7 +1102,7 @@ void ui_init (void)
       se_defuns (gse, jsobjs[i], defs);
       se_prop_create_xine_param (gse, jsobjs[i], "v",
 				 ranges[i].param, SE_TYPE_INT);
-      se_prop_add_listener (gse, jsobjs[i], "v", ui_listener, &ranges[i]);
+      se_prop_add_listener (gse, jsobjs[i], "v", ui_listener, (void *)&ranges[i]);
       se_prop_create_int (gse, jsobjs[i], "min", ranges[i].min, TRUE);
       se_prop_create_int (gse, jsobjs[i], "max", ranges[i].max, TRUE);
     }
@@ -1111,7 +1112,7 @@ void ui_init (void)
 gboolean ui_post_init (void)
 {
   unsigned int i;
-  uint32_t cap = gtk_video_get_capabilities (gtv);
+  uint32_t cap = gtk_video_get_capabilities ((GtkVideo *)gtv);
   gdk_threads_enter ();
   for (i = 0; i < G_N_ELEMENTS (ranges); ++i)
   {
